package com.xhkjedu.sexam.service.report;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.sexam.mapper.exam.EClassMapper;
import com.xhkjedu.sexam.mapper.paperstudent.EPaperStudentMapper;
import com.xhkjedu.sexam.mapper.report.ERbaseMapper;
import com.xhkjedu.sexam.mapper.report.EReportGenerateMapper;
import com.xhkjedu.sexam.mapper.report.ERsubjectMapper;
import com.xhkjedu.sexam.mapper.reportclass.ERclassMapper;
import com.xhkjedu.sexam.mapper.reportclass.ERclassQtypeMapper;
import com.xhkjedu.sexam.mapper.reportclass.ERclassQuestionErrorMapper;
import com.xhkjedu.sexam.mapper.reportclass.ERclassQuestionMapper;
import com.xhkjedu.sexam.mapper.reportclass.ERclassQuestionPointMapper;
import com.xhkjedu.sexam.mapper.reportstu.ERstudentMapper;
import com.xhkjedu.sexam.model.report.ERbase;
import com.xhkjedu.sexam.model.report.ERsubject;
import com.xhkjedu.sexam.model.reportclass.ERclass;
import com.xhkjedu.sexam.model.reportclass.ERclassQtype;
import com.xhkjedu.sexam.model.reportclass.ERclassQuestion;
import com.xhkjedu.sexam.model.reportclass.ERclassQuestionError;
import com.xhkjedu.sexam.model.reportclass.ERclassQuestionPoint;
import com.xhkjedu.sexam.model.reportstu.ERstudent;
import com.xhkjedu.sexam.utils.ExamUtil;
import com.xhkjedu.sexam.vo.paper.EPaperQsVo;
import com.xhkjedu.sexam.vo.report.ERPaperVo;
import com.xhkjedu.utils.N_Utils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.DoubleSummaryStatistics;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author ywx
 * @classname EReportGenerateService
 * @description 考试报告生成
 * @date 2022/8/8 10:04
 **/
@Service
public class EReportGenerateService {
    @Resource
    private EReportGenerateMapper eReportGenerateMapper;
    @Resource
    private EClassMapper eClassMapper;
    @Resource
    private EPaperStudentMapper ePaperStudentMapper;

    @Resource
    private ERbaseMapper eRbaseMapper;
    @Resource
    private ERstudentMapper eRstudentMapper;
    @Resource
    private ERsubjectMapper eRsubjectMapper;
    @Resource
    private ERclassMapper eRclassMapper;
    @Resource
    private ERclassQtypeMapper eRclassQtypeMapper;
    @Resource
    private ERclassQuestionMapper eRclassQuestionMapper;
    @Resource
    private ERclassQuestionPointMapper eRclassQuestionPointMapper;
    @Resource
    private ERclassQuestionErrorMapper eRclassQuestionErrorMapper;

    //生成考试报告
    public void generateExamReport(Integer examid) {
        List<Map> classes = eClassMapper.listByExamId(examid);
        List<Map> subjects = eReportGenerateMapper.listSubject(examid);
        List<ERPaperVo> papers = eReportGenerateMapper.listPaper(examid);
        int classnum = classes.size();
        int stunum = classes.stream().mapToInt(m -> (int) m.get("classnum")).sum();
        int subjectnum = subjects.size();

        String subjectid;
        String subjectname;
        Map<String, String> subjectMap = subjects.stream().collect(Collectors.toMap(m -> m.get("subjectid").toString(),
                m -> m.get("subjectname").toString()));
        Map<String, ERPaperVo> paperMap = papers.stream().collect(Collectors.toMap(m -> m.getSubjectid(), m -> m));

        List<ERstudent> schoolStudents = new ArrayList<>();//学校学生成绩
        List<ERstudent> schoolSubjectStudents = new ArrayList<>();//学校科目学生成绩
        List<ERstudent> classStudents = new ArrayList<>();//班级学生成绩
        List<ERstudent> classSubjectStudents = new ArrayList<>();//班级科目学生成绩
        Map<String, Integer> schoolStuRank = new HashMap<>();//学校学生分数排名
        Map<String, Integer> schoolSubStuRank = new HashMap<>();//学校科目学生分数排名
        Map<String, Integer> classStuRank = new HashMap<>();//班级学生分数排名
        Map<String, Integer> classSubStuRank = new HashMap<>();//班级科目学生分数排名
        List<ERstudent> students = ePaperStudentMapper.listByExamId(examid);
        Map<Integer, List<ERstudent>> schoolCollect = students.stream().collect(Collectors.groupingBy(s -> s.getStudentid()));
        for (Map.Entry<Integer, List<ERstudent>> entry : schoolCollect.entrySet()) {
            Integer studentid = entry.getKey();
            ERstudent cs = new ERstudent();
            cs.setStudentid(studentid);
            cs.setScore(entry.getValue().stream().mapToDouble(s -> s.getScore()).sum());
            schoolStudents.add(cs);
        }
        Map<Integer, List<ERstudent>> cCollect = students.stream().collect(Collectors.groupingBy(s -> s.getClassid()));
        for (Map.Entry<Integer, List<ERstudent>> entry : cCollect.entrySet()) {
            Integer classid = entry.getKey();
            subjectid = "zf";
            subjectname = "总分";
            List<ERstudent> studentList = entry.getValue();

            Map<Integer, List<ERstudent>> sCollect = studentList.stream().collect(Collectors.groupingBy(s -> s.getStudentid()));
            for (Map.Entry<Integer, List<ERstudent>> csEntry : sCollect.entrySet()) {
                ERstudent cs = new ERstudent();
                cs.setClassid(classid);
                cs.setStudentid(csEntry.getKey());
                cs.setSubjectid(subjectid);
                cs.setSubjectname(subjectname);
                cs.setScore(csEntry.getValue().stream().mapToDouble(s -> s.getScore()).sum());
                classStudents.add(cs);
            }
            Map<String, List<ERstudent>> subCollect = studentList.stream().collect(Collectors.groupingBy(s -> s.getSubjectid()));
            for (Map.Entry<String, List<ERstudent>> ssEntry : subCollect.entrySet()) {
                subjectid = ssEntry.getKey();
                subjectname = subjectMap.get(subjectid);
                studentList = ssEntry.getValue();
                for (ERstudent cs : studentList) {
                    cs.setSubjectname(subjectname);
                    classSubjectStudents.add(cs);
                }
            }
        }

        DoubleSummaryStatistics schoolStatistics = schoolStudents.stream().mapToDouble(s -> s.getScore()).summaryStatistics();
        DoubleSummaryStatistics classStatistics = classStudents.stream().mapToDouble(s -> s.getScore()).summaryStatistics();
        DoubleSummaryStatistics cSStatistics = classSubjectStudents.stream().mapToDouble(s -> s.getScore()).summaryStatistics();
        double schoolStatisticsMax = schoolStatistics.getMax();
        double schoolStatisticsAverage = schoolStatistics.getAverage();
        double schoolStatisticsMin = schoolStatistics.getMin();

        ERbase rbase = new ERbase();
        rbase.setExamid(examid);
        rbase.setClassnum(classnum);
        rbase.setStunum(stunum);
        rbase.setMissnum(students.stream().filter(l -> l.getSstate() == 3)
                .map(l -> l.getStudentid()).distinct().collect(Collectors.toList()).size());
        rbase.setSubjectnum(subjectnum);
        rbase.setMaxscore(schoolStatisticsMax);
        rbase.setAvgscore(schoolStatisticsAverage);
        rbase.setMixscore(schoolStatisticsMin);
        eRbaseMapper.insertSelective(rbase);

        Map<String, List<ERstudent>> subCollect = students.stream().collect(Collectors.groupingBy(s -> s.getSubjectid()));
        List<ERsubject> subjectList = new ArrayList<>();//考试报告科目情况
        List<ERclass> classList = new ArrayList<>();//考试报告班级情况
        List<ERclassQtype> qtypeList = new ArrayList<>();//考试报告班级题型情况
        List<ERclassQuestion> questionList = new ArrayList<>();//考试报告班级试题情况
        List<ERclassQuestionError> errorList = new ArrayList<>();//考试报告班级高频错题情况
        List<ERclassQuestionPoint> pointList = new ArrayList<>();//考试报告班级试题知识点情况
        for (Map.Entry<String, List<ERstudent>> ssEntry : subCollect.entrySet()) {
            subjectid = ssEntry.getKey();
            subjectname = subjectMap.get(subjectid);
            List<ERstudent> list = ssEntry.getValue();
            Map<Integer, List<ERstudent>> sCollect = list.stream().collect(Collectors.groupingBy(s -> s.getStudentid()));
            for (Map.Entry<Integer, List<ERstudent>> csEntry : sCollect.entrySet()) {
                ERstudent cs = new ERstudent();
                cs.setClassid(0);
                cs.setStudentid(csEntry.getKey());
                cs.setSubjectid(subjectid);
                cs.setSubjectname(subjectname);
                cs.setScore(csEntry.getValue().stream().mapToDouble(s -> s.getScore()).sum());
                schoolSubjectStudents.add(cs);
            }
            ERPaperVo rp = paperMap.get(subjectid);
            Double allscore = rp.getPscore();//满分
            double yxscore = allscore * 0.85;//优秀分
            double lhscore = allscore * 0.7;//良好分
            double jgscore = allscore * 0.6;//及格分
            double dfscore = allscore * 0.4;//低分

            DoubleSummaryStatistics statistics = list.stream().mapToDouble(s -> s.getScore()).summaryStatistics();
            ERsubject rs = new ERsubject();
            rs.setExamid(examid);
            rs.setSubjectid(subjectid);
            rs.setSubjectname(subjectname);
            rs.setScore(allscore);
            rs.setMaxscore(statistics.getMax());
            rs.setMinscore(statistics.getMin());
            rs.setAvgscore(statistics.getAverage());
            int yxnum = list.stream().filter(l -> l.getScore().compareTo(yxscore) != -1)
                    .collect(Collectors.toList()).size();
            int lhnum = list.stream().filter(l -> l.getScore().compareTo(lhscore) != -1)
                    .collect(Collectors.toList()).size();
            int jgnum = list.stream().filter(l -> l.getScore().compareTo(jgscore) != -1)
                    .collect(Collectors.toList()).size();
            int dfnum = list.stream().filter(l -> l.getScore().compareTo(dfscore) != 1)
                    .collect(Collectors.toList()).size();
            List<Double> scores = list.stream().map(l -> l.getScore()).collect(Collectors.toList());
            rs.setYxrate(N_Utils.getIntegerDivideAndMulitiply(yxnum, stunum));
            rs.setLhrate(N_Utils.getIntegerDivideAndMulitiply(lhnum, stunum));
            rs.setJgrate(N_Utils.getIntegerDivideAndMulitiply(jgnum, stunum));
            rs.setDfrate(N_Utils.getIntegerDivideAndMulitiply(dfnum, stunum));
            rs.setBzc(ExamUtil.standardDiviation(scores));
            int sknum = list.size();
            rs.setStunum(sknum);//实考人数
            rs.setMissnum(stunum - sknum);//缺考人数=计划人数-实考人数
            subjectList.add(rs);

            Map<Integer, List<ERstudent>> claCollect = list.stream().collect(Collectors.groupingBy(s -> s.getClassid()));
            for (Map.Entry<Integer, List<ERstudent>> entry : claCollect.entrySet()) {
                Integer classid = entry.getKey();
                List<ERstudent> rstudentList = entry.getValue();
                DoubleSummaryStatistics claStatistics = rstudentList.stream().mapToDouble(s -> s.getScore()).summaryStatistics();
                ERclass rc = new ERclass();
                rc.setExamid(examid);
                rc.setClassid(classid);
                rc.setSubjectid(subjectid);
                rc.setSubjectname(subjectname);
                rc.setAllscore(allscore);
                rc.setClassmaxscore(claStatistics.getMax());
                rc.setClassminscore(claStatistics.getMin());
                rc.setClassavgscore(claStatistics.getAverage());
                rc.setSchoolmaxscore(statistics.getMax());
                rc.setSchoolavgscore(statistics.getAverage());
                sknum = rstudentList.size();
                rc.setStunum(sknum);//实考人数
                rc.setMissnum(stunum - sknum);//缺考人数=计划人数-实考人数
                yxnum = rstudentList.stream().filter(l -> l.getScore().compareTo(yxscore) != -1)
                        .collect(Collectors.toList()).size();
                lhnum = rstudentList.stream().filter(l -> l.getScore().compareTo(lhscore) != -1)
                        .collect(Collectors.toList()).size();
                jgnum = rstudentList.stream().filter(l -> l.getScore().compareTo(jgscore) != -1)
                        .collect(Collectors.toList()).size();
                dfnum = rstudentList.stream().filter(l -> l.getScore().compareTo(dfscore) != 1)
                        .collect(Collectors.toList()).size();
                scores = rstudentList.stream().map(l -> l.getScore()).collect(Collectors.toList());
                rc.setYxrate(N_Utils.getIntegerDivideAndMulitiply(yxnum, stunum));
                rc.setLhrate(N_Utils.getIntegerDivideAndMulitiply(lhnum, stunum));
                rc.setJgrate(N_Utils.getIntegerDivideAndMulitiply(jgnum, stunum));
                rc.setDfrate(N_Utils.getIntegerDivideAndMulitiply(dfnum, stunum));
                rc.setBzc(ExamUtil.standardDiviation(scores));
                classList.add(rc);
                List<EPaperQsVo> typelist = JSON.parseArray(rp.getQnumjson(), EPaperQsVo.class);
                for (EPaperQsVo t : typelist) {
                    ERclassQtype rt = new ERclassQtype();
                    rt.setExamid(examid);
                    rt.setClassid(classid);
                    rt.setSubjectid(subjectid);
                    rt.setQtypename(t.getQtname());
                    rt.setScore(t.getScore());
                    rt.setNum(t.getNum());
                    rt.setSrate(t.getSrate());
                    qtypeList.add(rt);
                }

                List<Map> points = JSON.parseArray(rp.getQnumjson(), Map.class);
                for (Map p : points) {
                    ERclassQuestionPoint rqp = new ERclassQuestionPoint();
                    rqp.setExamid(examid);
                    rqp.setClassid(classid);
                    rqp.setSubjectid(subjectid);
                    rqp.setPointname(p.get("pointname").toString());
                    rqp.setScore((Double) p.get("score"));
                    rqp.setSrate((Double) p.get("srate"));
                    pointList.add(rqp);
                }
            }
        }

        eRclassMapper.insertList(classList);
        eRsubjectMapper.insertList(subjectList);

        setRank(schoolStudents, schoolStuRank, 1);//学校学生分数排名
        setRank(schoolSubjectStudents, schoolSubStuRank, 2);//学校科目学生分数排名
        setRank(classStudents, classStuRank, 3);//班级学生分数排名
        setRank(classSubjectStudents, classSubStuRank, 4);//班级科目学生分数排名

        for (ERstudent cs : classStudents) {
            Integer studentid = cs.getStudentid();
            cs.setClassrank(classStuRank.get(cs.getClassid() + "_" + studentid));
            cs.setClassmaxscore(classStatistics.getMax());
            cs.setClassavgscore(classStatistics.getAverage());
            cs.setSchoolrank(schoolStuRank.get(studentid));
            cs.setSchoolmaxscore(schoolStatisticsMax);
            cs.setSchoolavgscore(schoolStatisticsAverage);
        }
        eRstudentMapper.insertList(classStudents);

        DoubleSummaryStatistics sSStatistics = schoolSubjectStudents.stream().mapToDouble(s -> s.getScore()).summaryStatistics();
        for (ERstudent cs : classSubjectStudents) {
            Integer studentid = cs.getStudentid();
            Integer classid = cs.getClassid();
            subjectid = cs.getSubjectid();
            cs.setClassrank(classSubStuRank.get(classid + "_" + subjectid + "_" + studentid));
            cs.setClassmaxscore(cSStatistics.getMax());
            cs.setClassavgscore(cSStatistics.getAverage());
            cs.setSchoolrank(schoolSubStuRank.get(subjectid + "_" + studentid));
            cs.setSchoolmaxscore(sSStatistics.getMax());
            cs.setSchoolavgscore(sSStatistics.getAverage());
        }
        eRstudentMapper.insertList(classSubjectStudents);
    }

    //分数排名
    private void setRank(List<ERstudent> students, Map<String, Integer> rank, Integer code) {
        students =
                students.stream().sorted((s1, s2) -> -Double.compare(s1.getScore(), s2.getScore())).collect(Collectors.toList());
        List<Map.Entry<Double, List<ERstudent>>> gslist2 = students.stream()
                .collect(Collectors.groupingBy(ERstudent::getScore)).entrySet()
                .stream().sorted((s1, s2) -> -Double.compare(s1.getKey(), s2.getKey())).collect(Collectors.toList());
        Integer index = 1;
        String key;
        for (Map.Entry<Double, List<ERstudent>> entry : gslist2) {
            for (ERstudent cgs : entry.getValue()) {
                Integer studentid = cgs.getStudentid();
                Integer classid = cgs.getClassid();
                String subjectid = cgs.getSubjectid();
                if (code == 1) {//学校学生
                    key = studentid.toString();
                } else if (code == 2) {//学校科目学生
                    key = subjectid + "_" + studentid;
                } else if (code == 3) {//班级学生
                    key = classid + "_" + studentid;
                } else {//班级科目学生
                    key = classid + "_" + subjectid + "_" + studentid;
                }
                rank.put(key, index);
            }
            index = index + entry.getValue().size();
        }
    }
}
